@@ -5,7 +5,7 @@ use crate::{
     registry::LookupSpan,
 };
 
-use std::fmt::{self, Write};
+use std::fmt;
 use tracing_core::{
     field::{self, Field},
     Event, Level, Subscriber,
@@ -24,13 +24,12 @@ pub struct Pretty {
 
 /// The [visitor] produced by [`Pretty`]'s [`MakeVisitor`] implementation.
 ///
-/// [visitor]: ../../field/trait.Visit.html
-/// [`DefaultFields`]: struct.DefaultFields.html
-/// [`MakeVisitor`]: ../../field/trait.MakeVisitor.html
+/// [visitor]: field::Visit
+/// [`MakeVisitor`]: crate::field::MakeVisitor
+#[derive(Debug)]
 pub struct PrettyVisitor<'a> {
-    writer: &'a mut dyn Write,
+    writer: Writer<'a>,
     is_empty: bool,
-    ansi: bool,
     style: Style,
     result: fmt::Result,
 }
@@ -106,20 +105,36 @@ where
         let meta = event.metadata();
         write!(&mut writer, "  ")?;
 
+        // if the `Format` struct *also* has an ANSI color configuration,
+        // override the writer...the API for configuring ANSI color codes on the
+        // `Format` struct is deprecated, but we still need to honor those
+        // configurations.
+        if let Some(ansi) = self.ansi {
+            writer = writer.with_ansi(ansi);
+        }
+
         self.format_timestamp(&mut writer)?;
 
-        let style = if self.display_level && self.ansi {
+        let style = if self.display_level && writer.has_ansi_escapes() {
             Pretty::style_for(meta.level())
         } else {
             Style::new()
         };
 
         if self.display_level {
-            write!(writer, "{} ", super::FmtLevel::new(meta.level(), self.ansi))?;
+            write!(
+                writer,
+                "{} ",
+                super::FmtLevel::new(meta.level(), writer.has_ansi_escapes())
+            )?;
         }
 
         if self.display_target {
-            let target_style = if self.ansi { style.bold() } else { style };
+            let target_style = if writer.has_ansi_escapes() {
+                style.bold()
+            } else {
+                style
+            };
             write!(
                 writer,
                 "{}{}{}: ",
@@ -128,14 +143,12 @@ where
                 target_style.infix(style)
             )?;
         }
-        let mut v = PrettyVisitor::new(&mut writer, true)
-            .with_style(style)
-            .with_ansi(self.ansi);
+        let mut v = PrettyVisitor::new(writer.by_ref(), true).with_style(style);
         event.record(&mut v);
         v.finish()?;
         writer.write_char('\n')?;
 
-        let dimmed = if self.ansi {
+        let dimmed = if writer.has_ansi_escapes() {
             Style::new().dimmed().italic()
         } else {
             Style::new()
@@ -174,11 +187,7 @@ where
             writer.write_char('\n')?;
         }
 
-        let bold = if self.ansi {
-            Style::new().bold()
-        } else {
-            Style::new()
-        };
+        let bold = writer.bold();
         let span = event
             .parent()
             .and_then(|id| ctx.span(id))
@@ -220,12 +229,8 @@ where
 }
 
 impl<'writer> FormatFields<'writer> for Pretty {
-    fn format_fields<R: RecordFields>(
-        &self,
-        mut writer: Writer<'writer>,
-        fields: R,
-    ) -> fmt::Result {
-        let mut v = PrettyVisitor::new(&mut writer, false);
+    fn format_fields<R: RecordFields>(&self, writer: Writer<'writer>, fields: R) -> fmt::Result {
+        let mut v = PrettyVisitor::new(writer, false);
         fields.record(&mut v);
         v.finish()
     }
@@ -236,8 +241,8 @@ impl<'writer> FormatFields<'writer> for Pretty {
         fields: &span::Record<'_>,
     ) -> fmt::Result {
         let empty = current.is_empty();
-        let mut writer = current.as_writer();
-        let mut v = PrettyVisitor::new(&mut writer, empty);
+        let writer = current.as_writer();
+        let mut v = PrettyVisitor::new(writer, empty);
         fields.record(&mut v);
         v.finish()
     }
@@ -268,7 +273,7 @@ impl<'a> MakeVisitor<Writer<'a>> for PrettyFields {
 
     #[inline]
     fn make_visitor(&self, target: Writer<'a>) -> Self::Visitor {
-        PrettyVisitor::new(target.writer, true).with_ansi(self.ansi)
+        PrettyVisitor::new(target.with_ansi(self.ansi), true)
     }
 }
 
@@ -281,11 +286,10 @@ impl<'a> PrettyVisitor<'a> {
     /// - `writer`: the writer to format to.
     /// - `is_empty`: whether or not any fields have been previously written to
     ///   that writer.
-    pub fn new(writer: &'a mut dyn Write, is_empty: bool) -> Self {
+    pub fn new(writer: Writer<'a>, is_empty: bool) -> Self {
         Self {
             writer,
             is_empty,
-            ansi: true,
             style: Style::default(),
             result: Ok(()),
         }
@@ -295,10 +299,6 @@ impl<'a> PrettyVisitor<'a> {
         Self { style, ..self }
     }
 
-    pub(crate) fn with_ansi(self, ansi: bool) -> Self {
-        Self { ansi, ..self }
-    }
-
     fn write_padded(&mut self, value: &impl fmt::Debug) {
         let padding = if self.is_empty {
             self.is_empty = false;
@@ -310,7 +310,7 @@ impl<'a> PrettyVisitor<'a> {
     }
 
     fn bold(&self) -> Style {
-        if self.ansi {
+        if self.writer.has_ansi_escapes() {
             self.style.bold()
         } else {
             Style::new()
@@ -379,26 +379,14 @@ impl<'a> field::Visit for PrettyVisitor<'a> {
 }
 
 impl<'a> VisitOutput<fmt::Result> for PrettyVisitor<'a> {
-    fn finish(self) -> fmt::Result {
-        write!(self.writer, "{}", self.style.suffix())?;
+    fn finish(mut self) -> fmt::Result {
+        write!(&mut self.writer, "{}", self.style.suffix())?;
         self.result
     }
 }
 
 impl<'a> VisitFmt for PrettyVisitor<'a> {
     fn writer(&mut self) -> &mut dyn fmt::Write {
-        self.writer
-    }
-}
-
-impl<'a> fmt::Debug for PrettyVisitor<'a> {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        f.debug_struct("PrettyVisitor")
-            .field("writer", &format_args!("<dyn fmt::Write>"))
-            .field("is_empty", &self.is_empty)
-            .field("result", &self.result)
-            .field("style", &self.style)
-            .field("ansi", &self.ansi)
-            .finish()
+        &mut self.writer
     }
 }
